@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 ## NOTE: both url/socket need to have the query string
 ## url-encoded (percent format, e.g. %C3) if unicode.
@@ -603,7 +603,7 @@ Net::API::Nominatim - Perl client for local or public OpenStreetMap Nominatim Ge
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =head1 !NOMINATIM!
 
@@ -1018,6 +1018,10 @@ L<https://cpanratings.perl.org/d/Net-API-Nominatim>
 
 L<https://metacpan.org/release/Net-API-Nominatim>
 
+=item * PerlMonks!
+
+L<https://perlmonks.org/>
+
 =back
 
 
