@@ -3,7 +3,7 @@ package Net::API::Nominatim::Model::Address;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Data::Structure::Util qw/unbless/;
 use Data::Roundtrip qw/perl2dump json2perl perl2json no-unicode-escape-permanently/;
@@ -18,7 +18,7 @@ use Net::API::Nominatim::Model::BoundingBox;
 #our (@EXPORT_OK, %EXPORT_TAGS);
 #BEGIN {
 #	@EXPORT_OK = qw/
-#		fromHash fromArrayOfArrays fromArray fromJSONHash
+#		fromHash fromArray fromJSONHash
 #		fromJSONArray
 #	/;
 #	%EXPORT_TAGS = ( all => [@EXPORT_OK] );
@@ -112,7 +112,9 @@ sub clone { return Net::API::Nominatim::Model::Address->new($_[0]->toHash()) }
 # including whether they are defined or not.
 sub equals {
 	my ($x, $y) = @_;
-	return 0 unless ref($x) eq ref($y);
+	return 0 unless ref($x) eq ref($y); # not same object type
+	return 1 if "$x" eq "$y"; # same pointer
+
 	LL:
 	for($x->fields){
 		if( $_ eq 'boundingbox' ){
@@ -163,7 +165,7 @@ sub toJSON {
 
 ############################################################
 ####
-####  Exportable Factory Functions (static, not OO methods)
+####  (non-)Exportable Factory Functions (static, not OO methods)
 ####
 ############################################################
 
@@ -190,6 +192,23 @@ sub fromHash {
 	return $dst;
 }
 
+# factory sub to return an ARRAY of new objects given an Array of
+# Hashes, each containing fields for a single address.
+# If the 2nd parameter (destination) is left out,
+# it will be created AS AN ARRAY, therefore acting also as a "factory method".
+# It returns the destination (which can be newly-created).
+sub fromArray {
+	my $src = $_[0];
+	my $dst = $_[1] // [];
+
+	for my $pana (@$src){
+		my $h = fromHash($pana);
+		if( ! defined $h ){ print STDERR __PACKAGE__."::fromHash(), line ".__LINE__." : error, call to ".'fromHash()'." has failed for an address item, part of the input ARRAY of addresses.\n"; return undef }
+		push @$dst, $h;
+	}
+	return $dst;
+}
+
 # factory sub to construct a new object given a JSON string
 # containing a HASH represeting a single address, like
 # "{street: etc. ...}".
@@ -228,7 +247,7 @@ sub fromJSONArray {
 	if( ref($p) ne 'ARRAY' ){ print STDERR "${src}\n\n".__PACKAGE__."::fromJSONArray(), line ".__LINE__." : error, input JSON is not an ARRAY but '".ref($p)."', see above.\n"; return undef }
 	for my $pana (@$p){
 		my $h = fromHash($pana);
-		if( ! defined $h ){ print STDERR __PACKAGE__."::fromJSONHash(), line ".__LINE__." : error, call to ".'fromHash()'." has failed for an address item, part of the input ARRAY of addresses.\n"; return undef }
+		if( ! defined $h ){ print STDERR __PACKAGE__."::fromJSONArray(), line ".__LINE__." : error, call to ".'fromHash()'." has failed for an address item, part of the input ARRAY of addresses.\n"; return undef }
 		push @$dst, $h;
 	}
 	return $dst;
@@ -259,7 +278,7 @@ Net::API::Nominatim::Model::Address - Storage class for the address data as retu
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 
 =head1 DESCRIPTION
@@ -281,80 +300,46 @@ Example usage:
 
     use Net::API::Nominatim::Model::Address;
 
-    # construct from a hash of parameters,
-    # the keys must be exactly these:
-    my $bbox = Net::API::Nominatim::Model::Address->new({
-        lat1 => 30.12, lon1 => 12.22, lat2 => 30.15, lon2 => 12.5
+    my $address = Net::API::Nominatim::Model::Address->new({
+	'lat' => ...
+	'lon' => ...
+	'name' => ...
+	...
     });
 
-    # construct from a JSON string like the one used and returned
-    # by Nominatim.
-    my $bbox = Net::API::Nominatim::Model::Address->new(
-        #  lat1, lat2,  lon1,  lon2
-	"[30.12, 30.15, 12.22, 12.5]"
-    );
+    # or use the Random factory
+    my $address = Net::API::Nominatim::Model::Address::fromRandom();
 
-    # construct from an ARRAY whose order is the same
-    # as the one used and returned by Nominatim.
-    my $bbox = Net::API::Nominatim::Model::Address->new(
-        #  lat1, lat2,  lon1,  lon2
-	[30.12, 30.15, 12.22, 12.5]
-    );
+    # or use the JSON factory
+    # this returns just a single address from a JSON hash
+    # Nominatim's /reverse (for reverse geocoding) returns such JSON hash
+    my $address = Net::API::Nominatim::Model::Address::fromJSONHash($jsonstr);
 
-    # construct from an ARRAY of ARRAYs of [lat,lon]
-    my $bbox = Net::API::Nominatim::Model::Address->new(
-        #  lat1, lon2,    lat2,  lon2
-	[[30.12, 12.22], [30.15, 12.5]]
-    );
+    # this returns an array of Address objects,
+    # Nominatim's /search returns such JSON array-of-hashes
+    my $addresses = Net::API::Nominatim::Model::Address::fromJSONArray($jsonstr);
 
-    # print as the last example
-    print $bbox->toString();
+    # stringify
+    print $address->toString();
 
-    # print as JSON like the one used by Nominatim
-    print $bbox->toJSON();
+    # print as JSON string like the one returned by Nominatim's /search
+    print $address->toJSON();
 
-    # additionally there are exportable factory subs
+    # additionally there are (non-)exportable factory subs
     # construct from a hash of parameters,
     # the keys must be exactly these:
-    my $bbox = Net::API::Nominatim::Model::Address::fromHash({
-        lat1 => 30.12, lon1 => 12.22, lat2 => 30.15, lon2 => 12.5
+    my $address = Net::API::Nominatim::Model::Address::fromHash({
+        ...
     });
 
-    # construct from a JSON string like the one used and returned
-    # by Nominatim.
-    my $bbox = Net::API::Nominatim::Model::Address::fromJSONHash(
-        #  lat1, lat2,  lon1,  lon2
-	"[30.12, 30.15, 12.22, 12.5]"
-    );
-
-    # construct from an ARRAY whose order is the same
-    # as the one used and returned by Nominatim.
-    my $bbox = Net::API::Nominatim::Model::Address::fromArray(
-        #  lat1, lat2,  lon1,  lon2
-	[30.12, 30.15, 12.22, 12.5]
-    );
-
-    # construct from an ARRAY of ARRAYs of [lat,lon]
-    my $bbox = Net::API::Nominatim::Model::Address::fromArrayOfArrays(
-        #  lat1, lon2,    lat2,  lon2
-	[[30.12, 12.22], [30.15, 12.5]]
-    );
 
 =head1 EXPORT
 
-This is an OO module but it exports some factory subs:
-
-=over 2
-
-=item * fromJSON
-
-=item * fromHash
-
-=item * fromJSONArray
-
-=item * fromRandom
-
-=back
+Nothing is exported because the sane choice for
+sub names makes them too common thus a clash is imminent
+or they must be of huge lengths in order to ensure
+uniqueness. TLDR, use the fully qualified sub name,
+like C<Net::API::Nominatim::Model::Address::fromRandom()>.
 
 =head1 METHODS
 
@@ -368,22 +353,55 @@ The optional parameter can be:
 
 =over 2
 
-=item * an ARRAY_REF of C<[lat1, lat2, lon1, lon2]>. The specific
-order is what the actual
-L<Nominatim|https://nominatim.openstreetmap.org/search>
-service returns.
+=item * a HASH_REF which must contain any of these fields
+which are stored under this name internally.
+Note that there are setters and getters for each of these fields.
 
-=item * an ARRAY_REF of arrays of C<[lat,lon]>, e.g.
-C<[ [lat1,lon1], [lat2,lon2] ]>.
+=over 2
 
-=item * a HASH_REF which must contain one to four items
-keyed on C<lat1, lat2, lon1, lon2>. Values for whatever missing
-keys will be set to C<0.0>.
+=item * C<place_id>
 
-=item * a JSON string of C<"[lat1, lat2, lon1, lon2]">.
-The specific order is what the actual 
-L<Nominatim|https://nominatim.openstreetmap.org/search> 
-service returns.
+=item * C<osm_type>
+
+=item * C<osm_id>
+
+=item * C<lat>
+
+=item * C<lon>
+
+=item * C<name>
+
+=item * C<type>
+
+=item * C<place_rank>
+
+=item * C<boundingbox>
+
+=item * C<category>
+
+=item * C<addresstype>
+
+=item * C<importance>
+
+=item * C<display_name>
+
+=item * C<licence>
+
+=back
+
+All missing values will be set to a blank string. Except
+for C<boundingbox> which will be left C<undef>.
+
+=item * a JSON string containing just one hash which
+represents just one Address with one or more fields
+as outlined above. This JSON string is usually returned
+by Nominatim's C</reverse>. Nominatim's C</search>
+returns a JSON string of an array of hashes and you
+can not use that here, obviously (hint: use factory method
+L<fromJSONArray>).
+
+=item * an L<Net::API::Nominatim::Model::Address> object
+which we will clone it into the returned new Address object.
 
 =back
 
@@ -391,32 +409,133 @@ service returns.
 
 The constructor will return C<undef> on failure
 which can happen only if the input JSON string specified
-does not validate as JSON.
+does not validate as JSON. It will not die.
 
-=head2 lat1
+=head2 C<toString>
 
-Setter and Getter for the latitude C<lat1> of the first vertex of the bounding box.
+C<print $address-E<gt>toString();>
 
-=head2 lat2
+It returns a stringified version of this object.
 
-Setter and Getter for the longitude C<lon1> of the first vertex of the bounding box.
+=head2 C<toJSON>
 
-=head2 lon1
+C<my $jsonstr = $address-E<gt>toJSON();>
 
-Setter and Getter for the latitude C<lat2> of the second vertex of the bounding box.
+It returns a JSON string containing a hash with all the fields of this object.
 
-=head2 lon2
+=head2 C<clone>
 
-Setter and Getter for the longitude C<lon2> of the second vertex of the bounding box.
+C<my $newaddress = $address-E<gt>clone();>
 
-=head2 toString
+It returns a totally new L<Net::API::Nominatim::Model::Address> object
+deep-cloned from current object.
 
-It returns the bounding box as this string C<"[ [lat1,lon1], [lat2,lon2] ]">.
+=head2 C<equals>
 
-=head2 toJSON
+C<my $yes = $address-E<gt>equals($anotheraddress);>
 
-It returns the bounding box as this JSON string C<"[ lat1, lat2, lon1, lon2 ]">.
+It compares current object to the input object and returns 1 if they are equal
+or 0 if they are not. Missing values (which are blank strings or undef objects)
+will also count in the comparison.
 
+=head2 C<randomise>
+
+C<$address-E<gt>randomise();>
+
+It overwrites all fields with random strings, including those which were blank.
+C<boundingbox> will also be randomised as per L<Net::API::Nominatim::Model::BoundingBox::randomise>.
+
+=head2 fromHash 
+
+C<my $address = Net::API::Nominatim::Model::Address::fromHash({...});>
+
+Factory method to create a new L<Net::API::Nominatim::Model::Address> object
+given an input C<HASH_REF> with one or more fields this object contains, mentioned
+in the L<constructor|new>.
+
+=head2 fromArray
+
+C<my $addressesARRAY = Net::API::Nominatim::Model::Address::fromArray([{...},{...},...]);>
+
+Factory method to create an C<ARRAY_REF> of L<Net::API::Nominatim::Model::Address> objects
+given an input C<ARRAY_REF> in which each item is a C<HASH_REF> containing the data
+for a single address. Its fields are mentioned
+in the L<constructor|new>.
+
+=head2 C<fields>
+
+It returns all the fields stored in an L<Net::API::Nominatim::Model::Address> object
+as an array.
+
+=head2 C<place_id>
+
+If a parameter is provided then it sets the value of field C<place_id> to it,
+otherwise it returns the value of field C<place_id>.
+
+=head2 C<osm_type>
+
+If a parameter is provided then it sets the value of field C<osm_type> to it,
+otherwise it returns the value of field C<osm_type>.
+
+=head2 C<osm_id>
+
+If a parameter is provided then it sets the value of field C<osm_id> to it,
+otherwise it returns the value of field C<osm_id>.
+
+=head2 C<lat>
+
+If a parameter is provided then it sets the value of field C<lat> to it,
+otherwise it returns the value of field C<lat>.
+
+=head2 C<lon>
+
+If a parameter is provided then it sets the value of field C<lon> to it,
+otherwise it returns the value of field C<lon>.
+
+=head2 C<name>
+
+If a parameter is provided then it sets the value of field C<name> to it,
+otherwise it returns the value of field C<name>.
+
+=head2 C<type>
+
+If a parameter is provided then it sets the value of field C<type> to it,
+otherwise it returns the value of field C<type>.
+
+=head2 C<place_rank>
+
+If a parameter is provided then it sets the value of field C<place_rank> to it,
+otherwise it returns the value of field C<place_rank>.
+
+=head2 C<boundingbox>
+
+If a parameter is provided then it sets the value of field C<boundingbox> to it,
+otherwise it returns the value of field C<boundingbox>.
+
+=head2 C<category>
+
+If a parameter is provided then it sets the value of field C<category> to it,
+otherwise it returns the value of field C<category>.
+
+=head2 C<addresstype>
+
+If a parameter is provided then it sets the value of field C<addresstype> to it,
+otherwise it returns the value of field C<addresstype>.
+
+=head2 C<importance>
+
+If a parameter is provided then it sets the value of field C<importance> to it,
+otherwise it returns the value of field C<importance>.
+
+=head2 C<display_name>
+
+If a parameter is provided then it sets the value of field C<display_name> to it,
+otherwise it returns the value of field C<display_name>.
+
+=head2 C<licence>
+
+If a parameter is provided then it sets the value of field C<licence> to it,
+otherwise it returns the value of field C<licence>.
 
 =head1 AUTHOR
 
@@ -454,6 +573,10 @@ L<https://cpanratings.perl.org/d/Net-API-Nominatim-Model-Address>
 
 L<https://metacpan.org/release/Net-API-Nominatim-Model-Address>
 
+=item * PerlMonks!
+
+L<https://perlmonks.org/>
+
 =back
 
 
