@@ -3,7 +3,7 @@ package Net::API::Nominatim::Model::BoundingBox;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Data::Roundtrip qw/json2perl no-unicode-escape-permanently/;
 
@@ -80,7 +80,9 @@ sub clone { return Net::API::Nominatim::Model::BoundingBox->new($_[0]->toArray()
 # it first checks if the types of objects are the same.
 sub equals {
 	my ($x, $y) = @_;
-	return 0 unless ref($x) eq ref($y);
+	return 0 unless ref($x) eq ref($y); # not same object type
+	return 1 if "$x" eq "$y"; # same pointer
+
 	for(keys %$x){
 		return 0 unless $x->{$_} == $y->{$_};
 	}
@@ -113,7 +115,7 @@ sub toJSON {
 
 ############################################################
 ####
-####  Exportable Factory Functions (static, not OO methods)
+####  (non-)Exportable Factory Functions (static, not OO methods)
 ####
 ############################################################
 
@@ -226,13 +228,13 @@ Net::API::Nominatim::Model::BoundingBox - Storage class for the bounding box as
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 
 =head1 DESCRIPTION
 
-Net::API::Nominatim::Model::BoundingBox provides a Class
-for storing a generic 2-dimensional bounding box
+L<Net::API::Nominatim::Model::BoundingBox> is an OO module
+which provides a class to store a generic 2-dimensional bounding box
 comprising of two latitude/longitude coordinates
 denoting its two opposite vertices, with assorted
 constructor, getters, setters and stringifiers.
@@ -240,7 +242,6 @@ constructor, getters, setters and stringifiers.
 It can be constructed empty whereas all coordinates are set to C<0.0>,
 or loaded with data passed in during construction.
 
-
 =head1 SYNOPSIS
 
 Example usage:
@@ -279,7 +280,7 @@ Example usage:
     # print as JSON like the one used by Nominatim
     print $bbox->toJSON();
 
-    # additionally there are exportable factory subs
+    # additionally there are (non-)exportable factory subs
     # construct from a hash of parameters,
     # the keys must be exactly these:
     my $bbox = Net::API::Nominatim::Model::BoundingBox::fromHash({
@@ -308,21 +309,12 @@ Example usage:
 
 =head1 EXPORT
 
-This is an OO module but it exports some factory subroutines (aka I<methods>):
+Nothing is exported because the sane choice for
+sub names makes them too common thus a clash is imminent
+or they must be of huge lengths in order to ensure
+uniqueness. TLDR, use the fully qualified sub name,
+like C<Net::API::Nominatim::Model::BoundingBox::fromRandom()>.
 
-=over 2
-
-=item * fromHash
-
-=item * fromArrayOfArrays
-
-=item * fromArray
-
-=item * fromJSON
-
-=item * fromRandom
-
-=back
 
 =head1 METHODS
 
@@ -385,6 +377,16 @@ It returns the bounding box as this string C<"[ [lat1,lon1], [lat2,lon2] ]">.
 
 It returns the bounding box as this JSON string C<"[ lat1, lat2, lon1, lon2 ]">.
 
+=head2 clone
+
+It returns a totally new L<Net::API::Nominatim::Model::BoundingBox> object
+deep-cloned from current object.
+
+=head2 equals
+
+It compares current object to the input object and returns 1 if they are equal
+or 0 if they are not. Missing values (which are blank strings or undef objects)
+will also count in the comparison.
 
 =head1 AUTHOR
 
@@ -422,6 +424,10 @@ L<https://cpanratings.perl.org/d/Net-API-Nominatim-Model-BoundingBox>
 
 L<https://metacpan.org/release/Net-API-Nominatim-Model-BoundingBox>
 
+=item * PerlMonks!
+
+L<https://perlmonks.org/>
+
 =back
 
 
